@@ -5,6 +5,13 @@ pub struct InitialContract {
     pub deployer: Option<String>,
 }
 
+#[derive(Clone, Debug)]
+pub struct InitialLink {
+    pub contract_id: String,
+    pub stacks_node_addr: Option<String>,
+    pub cache: Option<String>,
+}
+
 #[derive(Clone, Debug, Serialize, Deserialize)]
 pub struct Account {
     pub address: String,
@@ -17,6 +24,7 @@ pub struct Account {
 #[derive(Clone, Debug, Default)]
 pub struct SessionSettings {
     pub include_boot_contracts: bool,
+    pub initial_links: Vec<InitialLink>,
     pub initial_contracts: Vec<InitialContract>,
     pub initial_accounts: Vec<Account>,
     pub initial_deployer: Option<Account>,
